unit maping;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, StdCtrls, Buttons, ComCtrls, CheckLst, Menus, Addition;

type

  { TEditorForm }

  TEditorForm = class(TForm)
    Image: TImage;
    MenuImages: TImageList;
    OpenDialog: TOpenDialog;
    TreeViewImages: TImageList;
    MainMenu: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    TreeView: TTreeView;
    procedure FormCreate(Sender: TObject);
    procedure ImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer
      );
    procedure ImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MenuItem11Click(Sender: TObject);
    procedure MenuItem2Click(Sender: TObject);
    procedure MenuItem3Click(Sender: TObject);
    procedure MenuItem4Click(Sender: TObject);
    procedure MenuItem6Click(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  private
    { private declarations }
    procedure LoadSurfaces;
    procedure LoadMap(file_name : string);
    procedure SaveMap;
    procedure DrawMap;
    procedure NewMap;
  public
    { public declarations }
  end; 

const
  nSurf =10; // число видов поверхности

var
  EditorForm: TEditorForm;
  px, py : integer;
  surfaces : array [0..nSurf] of TBitmap;
  map : array[1..17,1..21] of integer;
  map_tmp : array[1..17,1..21] of integer;
  can_draw : boolean;
  btn_draw : byte;
  WLS : string;

implementation

{ TEditorForm }

procedure TEditorForm.LoadSurfaces;
var i : integer;
begin
  for i:=0 to nSurf do
  begin
    surfaces[i]:=TBitmap.Create;
    surfaces[i].LoadFromFile('images'+WLS+'surfaces'+WLS+IntToStr(i)+'.bmp');
  end;
end;

procedure TEditorForm.NewMap;
var i, j : integer;
begin
  for j:=1 to 17 do
  for i:=1 to 21 do
   map_tmp[j,i]:=0;
end;

procedure TEditorForm.LoadMap(file_name : string);
var i, j : integer;
    f : textfile;
begin
  AssignFile(f,file_name);
  Reset(f);
  for j:=1 to 17 do
  begin
    for i:=1 to 21 do
    Read(f,map[j,i]);
    ReadLn(f);
  end;
  CloseFile(f);
  map_tmp:=map;
end;

procedure TEditorForm.SaveMap;
var i, j : integer;
    f : textfile;
begin
  map:=map_tmp;
  AssignFile(f,'maps'+WLS+'0.map');
  ReWrite(f);
  for j:=1 to 17 do
  begin
    for i:=1 to 21 do
    Write(f,map[j,i],' ');
    WriteLn(f);
  end;
  CloseFile(f);
end;

{procedure TEditorForm.DrawGrid;
var i, j : integer;
begin
  image.Canvas.pen.color:=clGreen;
  image.Canvas.brush.Color:=clWhite;
  with Image.Canvas do
  for j:=1 to 17 do
   begin
    for i:=1 to 21 do
    if map[j,i]=0 then
      begin brush.color:=clWhite; Rectangle((i-1)*30,(j-1)*30,(i-1)*30+30,(j-1)*30+30); end
    else
      begin brush.color:=clGray; Rectangle((i-1)*30,(j-1)*30,(i-1)*30+30,(j-1)*30+30); end;
   end;
   Image.Canvas.Pixels[0,0]:=clRed;
end;}

procedure TEditorForm.FormCreate(Sender: TObject);
begin
  {$IFDEF linux}
    WLS:='/';
  {$ENDIF}
  {$IFDEF windows}
   WLS:='\';
  {$ENDIF}
  LoadSurfaces;
  NewMap;
  DrawMap;
  can_draw:=false;
  TreeView.Items.Item[1].Selected:=true;
end;

procedure TEditorForm.ImageMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var px, py : integer;
begin
  if Button=mbRight then btn_draw:=0 else btn_draw:=1;
  can_draw:=true;
  px:=(X div 30)+1;
  py:=(Y div 30)+1;
  case btn_draw of
    1 :  map_tmp[py,px]:=TreeView.Selected.AbsoluteIndex;
    0 :  map_tmp[py,px]:=0;
  end;
  DrawMap;
end;

procedure TEditorForm.ImageMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
   if can_draw then
   begin
     px:=(X div 30)+1;
     py:=(Y div 30)+1;
     case btn_draw of
       1 :  map_tmp[py,px]:=TreeView.Selected.AbsoluteIndex;
       0 :  map_tmp[py,px]:=0;
     end;
     DrawMap;
   end;
end;

procedure TEditorForm.ImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  can_draw:=false;
end;

procedure TEditorForm.MenuItem11Click(Sender: TObject);
begin
  AboutForm.ShowModal;
end;

procedure TEditorForm.MenuItem2Click(Sender: TObject);
begin
  NewMap;
  DrawMap;
end;

procedure TEditorForm.MenuItem3Click(Sender: TObject);
begin
  if OpenDialog.Execute then
   LoadMap(OpenDialog.FileName);
  DrawMap;
end;

procedure TEditorForm.MenuItem4Click(Sender: TObject);
begin
  SaveMap;
end;

procedure TEditorForm.MenuItem6Click(Sender: TObject);
begin
  Close;
end;

procedure TEditorForm.SpeedButton1Click(Sender: TObject);
begin
end;

procedure TEditorForm.DrawMap;
var i, j : integer;
begin
  with Image.Canvas do
  for j:=1 to 17 do
   for i:=1 to 21 do
    Draw((i-1)*30,(j-1)*30,surfaces[map_tmp[j,i]]);
end;

initialization
  {$I maping.lrs}

end.

